@@ -11,5 +11,5 @@ class SortKey(NamedTuple):
     """Sort key."""
 
     is_index: bool
-    position: int
+    position: str
     title: str
