@@ -6,29 +6,32 @@ from enum import Enum, auto
 from functools import reduce
 from pathlib import Path
 from types import MappingProxyType
-from typing import Dict, Iterable, Iterator, Optional, Type
+from typing import Dict, Iterable, Optional, Type, Any
 
-import owlrl
 import rdflib
+from iolanta.models import LDContext
+from octadocs.conversions import triples_to_quads
+from octadocs.describe_documentation_page import describe_documentation_page
 from octadocs.octiron.context import merge
 from octadocs.octiron.context_loaders import (
     context_from_json,
     context_from_yaml,
 )
+from octadocs.octiron.inference import (
+    apply_inference_owlrl,
+    apply_inference_sparql,
+)
 from octadocs.octiron.plugins import (
     Loader,
     MarkdownLoader,
     TurtleLoader,
     YAMLLoader,
 )
-from octadocs.octiron.types import (
-    DEFAULT_CONTEXT,
-    DEFAULT_NAMESPACES,
-    Context,
-    Quad,
-    Triple, OCTA,
-)
-from urlpath import URL
+from octadocs.query import _format_query_bindings
+from octadocs.types import Context, QueryResult
+from octadocs.types import DEFAULT_NAMESPACES
+from octadocs.types import LOCAL as local
+from rdflib.plugins.sparql.processor import SPARQLResult
 
 if sys.version_info >= (3, 8):
     from functools import cached_property  # noqa
@@ -43,29 +46,6 @@ CONTEXT_FORMATS = MappingProxyType({
 })
 
 
-CLEAR_DEFAULT_QUERY = '''
-DELETE { ?s ?p ?o } WHERE {
-    ?s ?p ?o .
-    FILTER NOT EXISTS {
-        GRAPH ?g {
-            ?s ?p ?o .
-        }
-    }
-}
-'''
-
-
-def triples_to_quads(
-    triples: Iterator[Triple],
-    graph: rdflib.URIRef,
-) -> Iterator[Quad]:
-    """Convert sequence of triples to sequence of quads."""
-    yield from (
-        triple.as_quad(graph)
-        for triple in triples
-    )
-
-
 class CacheStatus(Enum):
     """Cache condition of a file."""
 
@@ -76,7 +56,12 @@ class CacheStatus(Enum):
 
 @dataclass
 class Octiron:   # noqa: WPS214
-    """Convert a lump of goo and data into a semantic graph."""
+    """
+    Convert a lump of goo and data into a semantic graph.
+
+    I believe that this class should be transformed into `Iolanta` class, and
+    factored out of `octadocs` application into its own package.
+    """
 
     root_directory: Path
     custom_namespaces: Dict[str, rdflib.Namespace] = field(default_factory=dict)
@@ -86,6 +71,7 @@ class Octiron:   # noqa: WPS214
             '__doc__': 'Time when every file was last imported into the graph.',
         },
     )
+    root_context: Optional[LDContext] = None
 
     @cached_property
     def namespaces(self):
@@ -120,7 +106,7 @@ class Octiron:   # noqa: WPS214
                 self._get_context_file,
                 reversed(list(self._find_context_files(directory))),
             ),
-            dict(DEFAULT_CONTEXT),
+            self.root_context or {},
         )
 
     def create_file_cache_status(
@@ -151,6 +137,7 @@ class Octiron:   # noqa: WPS214
         self,
         path: Path,
         local_iri: rdflib.URIRef,
+        named_contexts: Optional[Dict[str, Any]] = None,
         global_url: Optional[str] = None,
     ) -> None:
         """Update the graph from file determined by given path."""
@@ -194,6 +181,7 @@ class Octiron:   # noqa: WPS214
             context=context,
             local_iri=local_iri,
             global_url=global_url,
+            named_contexts=named_contexts,
         )
         triples = loader_instance.stream()
 
@@ -201,100 +189,49 @@ class Octiron:   # noqa: WPS214
 
         self.graph.addN(quads)
 
-        self.add_metadata(
-            local_iri=local_iri,
+        triples = describe_documentation_page(
+            iri=local_iri,
+            path=path,
+            docs_dir=self.root_directory,
         )
 
+        quads = triples_to_quads(
+            triples=triples,
+            graph=local.fs,
+        )
+
+        self.graph.addN(quads)
+
         # Store the file last modification time
         self.last_modified_timestamp_per_file[local_iri] = (
             file_last_modification_time
         )
 
-    def add_metadata(self, local_iri: rdflib.URIRef) -> None:
-        """Store metadata about a source file."""
-        local_url = URL(local_iri.toPython())
-        parent_iri = rdflib.URIRef(str(local_url.parent))
-        file_name = rdflib.Literal(local_url.name)
-        directory_name = rdflib.Literal(local_url.parent.name)
-
-        quads = [
-            (
-                local_iri,
-                rdflib.RDF.type,
-                OCTA.Directory,
-                local_iri,
-            ),
-            (
-                parent_iri,
-                OCTA.isParentOf,
-                local_iri,
-                local_iri,
-            ),
-            (
-                parent_iri,
-                OCTA.fileName,
-                directory_name,
-                local_iri,
-            ),
-            (
-                local_iri,
-                OCTA.fileName,
-                file_name,
-                local_iri,
-            ),
-        ]
-
-        self.graph.addN(quads)
-
-    def clear_default_graph(self) -> None:
-        """Remove all triples from the default graph."""
-        # Cannot use CLEAR DEFAULT due to:
-        #     https://github.com/RDFLib/rdflib/issues/1275
-        self.graph.update(CLEAR_DEFAULT_QUERY)
-
     def apply_inference(self) -> None:  # noqa: WPS213
-        """Do whatever is needed after the graph was updated from a file."""
-        # FIXME this should be customizable via dependency inversion. Right now
-        #   this is hardcoded to run inference rules formulated as SPARQL files.
-        logger.info('Clearing the unnamed inference graph')
-        self.clear_default_graph()
+        """Apply inference rules after graph has been updated from files."""
+        apply_inference_owlrl(graph=self.graph)
 
-        logger.info('Inference: OWL RL')
-        owlrl.DeductiveClosure(owlrl.OWLRL_Extension).expand(self.graph)
-
-        # Fill in octa:about relationships.
-        logger.info(
-            'Inference: '
-            '?thing octa:subjectOf ?page ⇒ ?page octa:about ?thing .',
+        apply_inference_sparql(
+            inference_directory=self.root_directory.parent / 'inference',
+            graph=self.graph,
         )
+
         self.graph.update('''
-            INSERT {
-                ?page octa:about ?thing .
-            } WHERE {
-                ?thing octa:subjectOf ?page .
+        INSERT {
+            GRAPH <local:inference/page-facet/> {
+                ?page octa:template ?template .
             }
+        } WHERE {
+            ?page iolanta:facet [
+                a octa:PageFacet ;
+                iolanta:supports octa: ;
+                octa:template ?template
+            ] .
+        }
         ''')
 
-        logger.info(
-            'Inference: ?thing rdfs:label ?label & '
-            '?thing octa:page ?page ⇒ ?page octa:title ?label',
-        )
-        self.graph.update('''
-            INSERT {
-                ?page octa:title ?title .
-            } WHERE {
-                ?subject
-                    rdfs:label ?title ;
-                    octa:subjectOf ?page .
-            }
-        ''')
-
-        inference_dir = self.root_directory.parent / 'inference'
-        if inference_dir.is_dir():
-            for sparql_file in inference_dir.iterdir():
-                logger.info('Inference: %s', sparql_file.name)
-                sparql_text = sparql_file.read_text()
-                self.graph.update(sparql_text)
+        # Changes made by SPARQL queries might require another pass of OWL RL
+        apply_inference_owlrl(graph=self.graph)
 
     def get_loader_class_for_path(self, path: Path) -> Optional[Type[Loader]]:
         """Based on file path, determine the loader to use."""
@@ -325,5 +262,29 @@ class Octiron:   # noqa: WPS214
 
     def _get_context_file(self, path: Path) -> Context:
         """Read and return context file by path."""
+        logger.info('Loading context: %s', path)
         context_loader = CONTEXT_FORMATS[path.name]
         return context_loader(path)
+
+    def query(
+        self,
+        query_text: str,
+        **kwargs: str,
+    ) -> QueryResult:
+        """Run SPARQL SELECT, CONSTRUCT, or ASK query."""
+        sparql_result: SPARQLResult = self.graph.query(
+            query_text,
+            initBindings=kwargs,
+        )
+
+        if sparql_result.askAnswer is not None:
+            return sparql_result.askAnswer
+
+        if sparql_result.graph is not None:
+            graph: rdflib.Graph = sparql_result.graph
+            for prefix, namespace in self.graph.namespaces():
+                graph.bind(prefix, namespace)
+
+            return graph
+
+        return _format_query_bindings(sparql_result.bindings)
