@@ -9,9 +9,15 @@ from typing import Any, Dict, Iterator, List, TypeVar, Union
 import rdflib
 from boltons.iterutils import remap
 from documented import DocumentedError
+from pyld.jsonld import JsonLdError
+
+from iolanta.models import LDContext, LDDocument
 from octadocs.octiron.context import merge
-from octadocs.octiron.types import LOCAL, Context, Triple
+from octadocs.pyld_document_loaders.choice import ChoiceLoader
+from octadocs.pyld_document_loaders.named import NamedLoader
+from octadocs.types import LOCAL, Context, Triple
 from pyld import jsonld
+from pyld.documentloader.requests import requests_document_loader
 
 try:  # noqa
     from yaml import CSafeDumper as SafeDumper  # noqa
@@ -29,17 +35,29 @@ Data = TypeVar('Data')
 @dataclass
 class ExpandError(DocumentedError):
     """
-    JSON-LD expand operation failed. Please review the source data below.
+    JSON-LD expand operation failed.
 
-    {self.formatted_data}
+    {self.message}
+
+    Data: {self.formatted_data}
+    Context: {self.formatted_context}
+    IRI: {self.iri}
     """
 
-    meta_data: MetaData
+    message: str
+    document: LDDocument
+    context: LDContext
+    iri: str
 
     @property
     def formatted_data(self) -> str:
-        """Format meta_data for printing."""
-        return pformat(self.meta_data, indent=4)
+        """Format document for printing."""
+        return json.dumps(self.document, indent=2)
+
+    @property
+    def formatted_context(self):
+        """Format context for printing."""
+        return json.dumps(self.context, indent=2)
 
 
 def _convert(term: Any) -> Any:  # type: ignore
@@ -65,6 +83,9 @@ def convert_dollar_signs(
 
     We use $ by convention to avoid writing quotes.
     """
+    if not meta_data:
+        return {}
+
     return remap(
         meta_data,
         lambda path, key, value: (  # noqa: WPS110
@@ -78,6 +99,8 @@ def as_triple_stream(
     raw_data: MetaData,
     context: Context,
     local_iri: str,
+    named_contexts: Dict[str, Any],
+    blank_node_prefix: str = '',
 ) -> Iterator[Triple]:
     """Convert YAML dict to a stream of triples."""
     if isinstance(raw_data, list):
@@ -85,6 +108,8 @@ def as_triple_stream(
             context=context,
             local_iri=local_iri,
             raw_data=raw_data,
+            named_contexts=named_contexts,
+            blank_node_prefix=blank_node_prefix,
         )
 
     elif isinstance(raw_data, dict):
@@ -92,6 +117,8 @@ def as_triple_stream(
             context=context,
             local_iri=local_iri,
             raw_data=raw_data,
+            named_contexts=named_contexts,
+            blank_node_prefix=blank_node_prefix,
         )
 
     else:
@@ -119,14 +146,24 @@ def reformat_blank_nodes(prefix: str, triple: Triple) -> Triple:
 
 
 def _dict_as_triple_stream(  # type: ignore
-    raw_data: Dict[str, Any],
-    context: Context,
+    raw_data: LDDocument,
+    context: LDContext,
     local_iri: str,
+    named_contexts: Dict[str, LDContext],
+    blank_node_prefix: str = '',
 ) -> Iterator[Triple]:
     """Convert dict into a triple stream."""
     meta_data = convert_dollar_signs(raw_data)
 
     local_context = meta_data.pop('@context', None)
+    if isinstance(local_context, str):
+        try:
+            local_context = named_contexts[local_context]
+        except KeyError as err:
+            raise ValueError(
+                f'Named context {local_context} is unknown.',
+            ) from err
+
     if local_context is not None:
         context = merge(
             first=context,
@@ -141,7 +178,7 @@ def _dict_as_triple_stream(  # type: ignore
 
     # The author specified an IRI the document tells us about. Let us
     # link this IRI to the local document IRI.
-    meta_data['octa:subjectOf'] = local_iri
+    meta_data['subjectOf'] = local_iri
 
     # Reason: https://github.com/RDFLib/rdflib-jsonld/issues/97
     # If we don't expand with an explicit @base, import will fail silently.
@@ -151,11 +188,18 @@ def _dict_as_triple_stream(  # type: ignore
             options={
                 'base': str(LOCAL),
                 'expandContext': context,
+                'documentLoader': ChoiceLoader(loaders=[
+                     NamedLoader(documents=named_contexts),
+                     requests_document_loader(),
+                ]),
             },
         )
-    except TypeError as err:
+    except (TypeError, JsonLdError) as err:
         raise ExpandError(
-            meta_data=meta_data,
+            message=str(err),
+            document=meta_data,
+            context=context,
+            iri=local_iri,
         ) from err
 
     # Reason: https://github.com/RDFLib/rdflib-jsonld/issues/98
@@ -171,7 +215,7 @@ def _dict_as_triple_stream(  # type: ignore
     yield from map(
         partial(
             reformat_blank_nodes,
-            f'{local_iri}/',
+            f'{local_iri}/{blank_node_prefix}',
         ),
         starmap(
             Triple,
@@ -184,11 +228,15 @@ def _list_as_triple_stream(  # type: ignore
     raw_data: List[Dict[str, Any]],
     context: Context,
     local_iri: str,
+    named_contexts: Dict[str, Any],
+    blank_node_prefix: str = '',
 ) -> Iterator[Triple]:
     """Convert a list into a triple stream."""
-    for sub_document in raw_data:
+    for sub_document_number, sub_document in enumerate(raw_data):
         yield from as_triple_stream(
             raw_data=sub_document,
             context=context,
             local_iri=local_iri,
+            named_contexts=named_contexts,
+            blank_node_prefix=f'{blank_node_prefix}_{sub_document_number}',
         )
