@@ -2,21 +2,28 @@ import logging
 import operator
 from functools import lru_cache, partial
 from pathlib import Path
-from typing import Callable, Optional
+from typing import Optional, Any, Dict, Callable
 
 import rdflib
-from livereload import Server
+from mkdocs.livereload import LiveReloadServer
 from mkdocs.plugins import BasePlugin
 from mkdocs.structure.files import Files
 from mkdocs.structure.nav import Navigation
 from mkdocs.structure.pages import Page
-from octadocs.environment import src_path_to_iri
+from octadocs.conversions import iri_by_page, src_path_to_iri
+from octadocs.default_context import construct_root_context
+from octadocs.iolanta import render
+from octadocs.macros import link
 from octadocs.navigation.processor import OctadocsNavigationProcessor
 from octadocs.octiron import Octiron
-from octadocs.octiron.types import LOCAL
-from octadocs.query import Query, query
+from octadocs.query import query
+from octadocs.storage import DiskCacheStorage
 from octadocs.stored_query import StoredQuery
+from octadocs.types import LOCAL, Query, OCTA, DEFAULT_NAMESPACES
 from typing_extensions import TypedDict
+from rich.traceback import install
+
+install(show_locals=False)
 
 logger = logging.getLogger(__name__)
 
@@ -27,6 +34,7 @@ class ConfigExtra(TypedDict):
     graph: rdflib.ConjunctiveGraph
     octiron: Octiron
     queries: StoredQuery
+    named_contexts: Dict[str, Any]
 
 
 class Config(TypedDict):
@@ -46,6 +54,7 @@ class TemplateContext(TypedDict):
     query: Query
     queries: StoredQuery
     local: rdflib.Namespace
+    render: Callable[[rdflib.URIRef], str]
 
     # FIXME this is hardcode and should be removed
     rdfs: rdflib.Namespace
@@ -74,7 +83,12 @@ def get_template_by_page(
 @lru_cache(None)
 def cached_octiron(docs_dir: Path) -> Octiron:
     """Retrieve cached Octiron instance or create it if absent."""
-    return Octiron(root_directory=docs_dir)
+    return Octiron(
+        root_directory=docs_dir,
+        root_context=construct_root_context(
+            namespaces=DEFAULT_NAMESPACES,
+        ),
+    )
 
 
 class OctaDocsPlugin(BasePlugin):
@@ -106,6 +120,7 @@ class OctaDocsPlugin(BasePlugin):
             'graph': self.octiron.graph,
             'octiron': self.octiron,
             'queries': self.stored_query,
+            'named_contexts': {},
         })
 
         return config
@@ -114,9 +129,18 @@ class OctaDocsPlugin(BasePlugin):
         """Extract metadata from files and compose the site graph."""
         # Load the Octadocs vocabulary into graph
         self.octiron.update_from_file(
-            path=Path(__file__).parent / 'octadocs.yaml',
-            local_iri=rdflib.URIRef('https://ns.octadocs.io/'),
+            path=Path(__file__).parent / 'yaml/octadocs.yaml',
+            local_iri=rdflib.URIRef(OCTA),
             global_url='/octadocs.yaml',
+            named_contexts=config['extra']['named_contexts'],
+        )
+
+        # And the global iolanta vocabulary
+        self.octiron.update_from_file(
+            path=Path(__file__).parent / 'yaml/iolanta.yaml',
+            local_iri=rdflib.URIRef('https://iolanta.tech/'),
+            global_url='/iolanta.yaml',
+            named_contexts=config['extra']['named_contexts'],
         )
 
         for mkdocs_file in files:
@@ -124,10 +148,13 @@ class OctaDocsPlugin(BasePlugin):
                 path=Path(mkdocs_file.abs_src_path),
                 local_iri=src_path_to_iri(mkdocs_file.src_path),
                 global_url=f'/{mkdocs_file.url}',
+                named_contexts=config['extra']['named_contexts'],
             )
 
         self.octiron.apply_inference()
 
+        DiskCacheStorage(octiron=self.octiron).save()
+
     def on_page_markdown(
         self,
         markdown: str,
@@ -136,6 +163,8 @@ class OctaDocsPlugin(BasePlugin):
         files: Files,
     ):
         """Inject page template path, if necessary."""
+        page.iri = iri_by_page(page)
+
         template_name = get_template_by_page(
             page=page,
             graph=self.octiron.graph,
@@ -182,27 +211,28 @@ class OctaDocsPlugin(BasePlugin):
             instance=self.octiron.graph,
         )
         context['queries'] = self.stored_query
-        context['local'] = LOCAL
 
-        # Provide all the support namespaces into template context
+        context['local'] = LOCAL
+        context['LOCAL'] = LOCAL
         context.update(self.octiron.namespaces)
 
+        context['render'] = partial(
+            render,
+            octiron=self.octiron,
+        )
+
+        # Provide all the support namespaces into template context
+        context['octiron'] = self.octiron
+        context['link'] = partial(
+            link,
+            octiron=self.octiron,
+        )
+
+        # Page attributes
+        page.iri = page_iri
+
         return context
 
-    def on_serve(
-        self,
-        server: Server,
-        config: Config,
-        builder: Callable,  # type: ignore
-    ) -> Server:
-        """Watch the stored queries directory if it exists."""
-        stored_queries = Path(config['docs_dir']).parent / 'queries'
-
-        if stored_queries.is_dir():
-            server.watch(str(stored_queries))
-
-        return server
-
     def on_nav(
         self,
         nav: Navigation,
@@ -217,3 +247,21 @@ class OctaDocsPlugin(BasePlugin):
             ).generate()
 
         return nav
+
+    def on_serve(
+        self,
+        server: LiveReloadServer,
+        config: Config,
+        builder,
+    ) -> LiveReloadServer:
+        inference_directory = Path(config['docs_dir']).parent / 'inference'
+        if inference_directory.is_dir():
+            for sparql_file in inference_directory.glob('**/*.sparql'):
+                server.watch(str(sparql_file), builder)
+
+        queries_directory = Path(config['docs_dir']).parent / 'queries'
+        if queries_directory.is_dir():
+            for sparql_file in queries_directory.glob('**/*.sparql'):
+                server.watch(str(sparql_file), builder)
+
+        return server
